@@ -4,10 +4,12 @@ import argparse
 import collections
 import copy
 import os
+import pathlib
 import re
 import subprocess
 import sys
 import warnings
+from typing import Any
 from typing import Hashable
 from typing import Iterator
 from typing import Mapping
@@ -16,6 +18,11 @@ from typing import NamedTuple
 from typing import NoReturn
 from typing import Sequence
 
+if sys.version_info >= (3, 11):  # pragma: no cover
+    import tomllib
+else:
+    import tomli as tomllib
+
 with warnings.catch_warnings():
     # DeprecationWarning: 'cgi' is deprecated and slated for
     # removal in Python 3.13
@@ -24,7 +31,6 @@ with warnings.catch_warnings():
     from Cython import Tempita
     from Cython.Compiler.TreeFragment import StringParseContext
 import Cython
-from Cython.Compiler.ExprNodes import GeneratorExpressionNode, CythonArrayNode
 from Cython.Compiler.ExprNodes import IndexNode
 from Cython.Compiler.ExprNodes import SimpleCallNode
 from Cython.Compiler.ExprNodes import AttributeNode
@@ -41,15 +47,12 @@ from Cython.Compiler.ExprNodes import FormattedValueNode
 from Cython.Compiler.ExprNodes import JoinedStrNode
 from Cython.Compiler.ExprNodes import ImportNode
 from Cython.Compiler.ExprNodes import NameNode
-from Cython.Compiler.ExprNodes import NewExprNode
 from Cython.Compiler.ExprNodes import LambdaNode
-from Cython.Compiler.ExprNodes import TypecastNode
 from Cython.Compiler.ModuleNode import ModuleNode
-from Cython.Compiler.Nodes import CArgDeclNode, MemoryViewSliceTypeNode
+from Cython.Compiler.Nodes import CArgDeclNode
 from Cython.Compiler.Nodes import AssertStatNode
 from Cython.Compiler.Nodes import IfClauseNode
 from Cython.Compiler.Nodes import StatListNode
-from Cython.Compiler.Nodes import CClassDefNode
 from Cython.Compiler.Nodes import CFuncDeclaratorNode
 from Cython.Compiler.Nodes import CFuncDefNode
 from Cython.Compiler.Nodes import GlobalNode
@@ -62,7 +65,6 @@ from Cython.Compiler.Nodes import ExprStatNode
 from Cython.Compiler.Nodes import ForInStatNode
 from Cython.Compiler.Nodes import FromCImportStatNode
 from Cython.Compiler.Nodes import FromImportStatNode
-from Cython.Compiler.Nodes import FusedTypeNode
 from Cython.Compiler.Nodes import Node
 from Cython.Compiler.Nodes import SingleAssignmentNode
 from Cython.Compiler.TreeFragment import parse_from_strings
@@ -73,6 +75,14 @@ from cython_lint import __version__
 
 CYTHON_VERSION = tuple(Cython.__version__.split('.'))
 
+EXCLUDES = (
+    r'/('
+    r'\.direnv|\.eggs|\.git|\.hg|\.ipynb_checkpoints|\.mypy_cache|\.nox|\.svn|'
+    r'\.tox|\.venv|'
+    r'_build|buck-out|build|dist|venv'
+    r')/'
+)
+
 if CYTHON_VERSION > ('3',):  # pragma: no cover
     from Cython.Compiler.ExprNodes import AnnotationNode
 else:  # pragma: no cover
@@ -106,6 +116,23 @@ PYCODESTYLE_CODES = frozenset((
 
 CONSTANT_NODE = (UnicodeNode, IntNode, FloatNode)
 
+MISSING_CHILD_ATTRS = frozenset((
+    'bases',
+    'decorators',
+    'base_type',
+    'loop',
+    'decorators',
+    'types',
+    'target',
+    'cppclass',
+    'args',
+    'result_expr',
+    'expr',
+    'attribute',
+    'base_type_node',
+    'annotation',
+))
+
 
 class NodeParent(NamedTuple):
     node: Node
@@ -192,7 +219,18 @@ def visit_funcdef(
         # e.g. import numpy as np
         and not isinstance(_child.rhs, ImportNode)
     ]
-    defs = [*defs, *simple_assignments]
+    tuple_assignments = []
+    for _child in children:
+        if (
+            isinstance(_child, SingleAssignmentNode)
+            and isinstance(_child.lhs, TupleNode)
+            # e.g. import numpy as np
+            and not isinstance(_child.rhs, ImportNode)
+        ):
+            for _arg in _child.lhs.args:
+                if isinstance(_arg, NameNode):
+                    tuple_assignments.append(Token(_arg.name, *_arg.pos[1:]))
+    defs = [*defs, *simple_assignments, *tuple_assignments]
 
     names = [
         Token(_child.name, *_child.pos[1:])
@@ -223,7 +261,8 @@ def visit_funcdef(
 
             violations.append((
                 _def[1], _def[2]+1,
-                f'\'{_def[0]}\' defined but unused',
+                f'\'{_def[0]}\' defined but unused '
+                '(try prefixing with underscore?)',
             ))
         if _def[0] in [_import[0] for _import in global_imports]:
             _global_import = [
@@ -688,7 +727,6 @@ def run_ast_checks(
             _code, filename, _lines, skip_check=True,
         )
         included_names.extend(_included_names)
-
     for _import in imported_names:
         if _import[0] == '*':
             continue
@@ -709,13 +747,15 @@ def run_ast_checks(
 
 
 def run_pycodestyle(
-    line_length: int, filename: str,
+    line_length: int,
+    filename: str,
     violations: list[tuple[int, int, str]],
+    ignore: set[str],
 ) -> None:
     output = subprocess.run(
         [
             'pycodestyle',
-            f'--ignore={",".join(PYCODESTYLE_CODES)}',
+            f'--ignore={",".join(PYCODESTYLE_CODES | ignore)}',
             f'--max-line-length={line_length}',
             '--format=%(row)d:%(col)d:%(code)s %(text)s',
             filename,
@@ -739,10 +779,14 @@ def _main(
     ext: str,
     line_length: int = 88,
     no_pycodestyle: bool = False,
+    ignore: set[str] | None = None,
 ) -> int:
+    if ignore is None:
+        ignore = set()
+    assert ignore is not None  # help mypy
     violations: list[tuple[int, int, str]] = []
     if not no_pycodestyle:
-        run_pycodestyle(line_length, filename, violations)
+        run_pycodestyle(line_length, filename, violations, ignore)
 
     lines = {}
     if ext == '.pyx':
@@ -773,32 +817,9 @@ def traverse(tree: ModuleNode) -> Iterator[NodeParent]:
             continue
 
         child_attrs = set(copy.deepcopy(node.child_attrs))
-
-        if isinstance(node, CClassDefNode):
-            child_attrs.update(['bases', 'decorators'])
-        elif isinstance(node, TypecastNode):
-            child_attrs.add('base_type')
-        elif isinstance(node, GeneratorExpressionNode):
-            if hasattr(node, 'loop'):
-                child_attrs.add('loop')
-            else:  # pragma: no cover
-                err_msg(node, 'GeneratorExpressionNode with loop attribute')
-        elif isinstance(node, CFuncDefNode):
-            child_attrs.add('decorators')
-        elif isinstance(node, FusedTypeNode):
-            child_attrs.add('types')
-        elif isinstance(node, ForInStatNode):
-            child_attrs.add('target')
-        elif isinstance(node, NewExprNode):
-            child_attrs.add('cppclass')
-        elif isinstance(node, LambdaNode):
-            child_attrs.update(['args', 'result_expr'])
-        elif isinstance(node, AnnotationNode):  # pragma: no cover
-            child_attrs.add('expr')
-        elif isinstance(node, AttributeNode):
-            child_attrs.add('attribute')
-        elif isinstance(node, (MemoryViewSliceTypeNode, CythonArrayNode)):
-            child_attrs.add('base_type_node')
+        for attr in MISSING_CHILD_ATTRS:
+            if hasattr(node, attr):
+                child_attrs.add(attr)
 
         for attr in child_attrs:
             child = getattr(node, attr)
@@ -809,26 +830,92 @@ def traverse(tree: ModuleNode) -> Iterator[NodeParent]:
         yield node_parent
 
 
+def _get_config(paths: list[pathlib.Path]) -> dict[str, Any]:
+    """Get the configuration from a config file
+
+    Search for a pyproject.toml file in common parent directories
+    of the given list of paths.
+    """
+    root = pathlib.Path(os.path.commonpath(paths))
+    root = root.parent if root.is_file() else root
+
+    while root != root.parent:
+
+        config_file = root / 'pyproject.toml'
+        if config_file.is_file():
+            config = tomllib.loads(config_file.read_text())
+            config = config.get('tool', {}).get('cython-lint', {})
+            if config:
+                return config
+
+        root = root.parent
+
+    return {}
+
+
 def main(argv: Sequence[str] | None = None) -> int:  # pragma: no cover
     parser = argparse.ArgumentParser()
     parser.add_argument('paths', nargs='*')
+    parser.add_argument(
+        '--files',
+        help='Regex pattern with which to match files to include',
+        required=False,
+        default=r'',
+    )
+    parser.add_argument(
+        '--exclude',
+        help='Regex pattern with which to match files to exclude',
+        required=False,
+        default=r'^$',
+    )
     # default from black formatter
     parser.add_argument('--max-line-length', type=int, default=88)
     parser.add_argument('--no-pycodestyle', action='store_true')
     parser.add_argument('--version', action='version', version=__version__)
+    parser.add_argument(
+        '--ignore',
+        nargs='*',
+        default='',
+        help='Comma-separated list of pycodestyle error codes to ignore',
+    )
     args = parser.parse_args(argv)
+    paths = [pathlib.Path(path).resolve() for path in args.paths]
+
+    # Update defaults from pyproject.toml if present
+    config = {k.replace('-', '_'): v for k, v in _get_config(paths).items()}
+    parser.set_defaults(**config)
+    args = parser.parse_args(argv)
+
     ret = 0
-    for path in args.paths:
-        _, ext = os.path.splitext(path)
-        try:
-            with open(path, encoding='utf-8') as fd:
-                content = fd.read()
-        except UnicodeDecodeError:
-            continue
-        ret |= _main(
-            content, path, line_length=args.max_line_length,
-            no_pycodestyle=args.no_pycodestyle, ext=ext,
-        )
+
+    if not isinstance(args.ignore, list):
+        args.ignore = [args.ignore]
+    ignore = {code.strip() for s in args.ignore for code in s.split(',')}
+
+    for path in paths:
+        if path.is_file():
+            filepaths = iter((path,))
+        else:
+            filepaths = (
+                p for p in path.rglob('*')
+                if re.search(args.files, str(p.as_posix()), re.VERBOSE)
+                and not re.search(args.exclude, str(p.as_posix()), re.VERBOSE)
+                and not re.search(EXCLUDES, str(p.as_posix()))
+                and p.suffix in ('.pyx', '.pxd', '.pxi')
+            )
+
+        for filepath in filepaths:
+            ext = filepath.suffix
+            try:
+                with open(filepath, encoding='utf-8') as fd:
+                    content = fd.read()
+            except UnicodeDecodeError:
+                continue
+            ret |= _main(
+                content, str(filepath), line_length=args.max_line_length,
+                no_pycodestyle=args.no_pycodestyle, ext=ext,
+                ignore=ignore,
+            )
     return ret
 
 
